// Harvest System Script v.0.2 by SickLab

using System;
using Server.Targeting;


namespace Server.Engines.Harvest
{
	public abstract class HarvestSystem
	{
		private int m_MinHarvestEffect;
		private int m_MaxHarvestEffect;
		private double m_Delay;

		private HarvestItemCol m_HarvestItems;
		private HarvestUsedCol m_HarvestUseds;
		private HarvestResDefCol m_HarvestResDefs;

		public int MinHarvestEffect { get { return m_MinHarvestEffect; } }
		public int MaxHarvestEffect { get { return m_MaxHarvestEffect; } }
		public double Delay { get { return m_Delay; } }

		public HarvestItemCol HarvestItems{ get { return m_HarvestItems; } }
		public HarvestUsedCol HarvestUseds{ get { return m_HarvestUseds; } }
		public HarvestResDefCol HarvestResDefs{ get { return m_HarvestResDefs; } }

		public abstract SkillName MainSkill{ get; }

		public abstract void InitHarvestList();
		public abstract void PlayHarvestEffect( Mobile from );
		public abstract void PlayEndingEffect( Mobile from, bool failed, bool toBackpack, bool noMoreRes, bool hasMovedAway );
		public abstract bool CanHarvest( Mobile from, Item item, HarvestItem harvestItem );
		
		public HarvestSystem( int minHarvestEffect, int maxHarvestEffect, double delay )
		{
			m_MinHarvestEffect = minHarvestEffect;
			m_MaxHarvestEffect = maxHarvestEffect;
			m_Delay = delay;

			m_HarvestItems = new HarvestItemCol();
			m_HarvestUseds = new HarvestUsedCol();
			m_HarvestResDefs = new HarvestResDefCol();

			InitHarvestList();
		}

		public void AddResourceInfo( Type type, int minTotalRes, int maxTotalRes, double minSkill, double maxSkill )
		{
			HarvestResDef harvestResDef = new HarvestResDef( type, minTotalRes, maxTotalRes, minSkill, maxSkill );
			m_HarvestResDefs.Add( harvestResDef );
		}

		public HarvestUsed SearchFor( Mobile from, int itemID, int x, int y, int z, HarvestItem harvestItem )
		{
			HarvestUsed harvestUsed = m_HarvestUseds.SearchFor( itemID, x, y, z );

			// If not founded, add it to the list
			if ( harvestUsed == null )
			{
				int iMin = harvestItem.MinTotalRes;
				int iMax = ( harvestItem.MaxTotalRes - iMin ) + 1;
				int iRandom = Utility.Random( iMax );
				iRandom += iMin;
				
				Type DefaultResType = harvestItem.TypeRes;
				Type SpecialResType = from.Region.GetResource( DefaultResType );
				harvestUsed = new HarvestUsed( itemID, x, y, z, iRandom, this, harvestItem.RegenDelay, DefaultResType, SpecialResType );
				m_HarvestUseds.Add( harvestUsed );
			}

			return harvestUsed;
		}

		public void AddHarvest( int itemID, int minTotalRes, int maxTotalRes, double minSkill, double maxSkill, Type typeRes, int minAmount, int maxAmount, TimeSpan regenDelay, int localizedMessage )
		{
			HarvestItem harvestItem = new HarvestItem( itemID, minTotalRes, maxTotalRes, MainSkill, minSkill, maxSkill, typeRes, minAmount, maxAmount, regenDelay, localizedMessage );
            m_HarvestItems.Add( harvestItem );
		}

		public void AddHarvest( int[] itemIDs, int minTotalRes, int maxTotalRes, double minSkill, double maxSkill, Type typeRes, int minAmount, int maxAmount, TimeSpan regenDelay, int localizedMessage )
		{
			for ( int i = 0; i < itemIDs.Length; i++ )
			{
				AddHarvest( itemIDs[i], minTotalRes, maxTotalRes, minSkill, maxSkill, typeRes, minAmount, maxAmount, regenDelay, localizedMessage );
			}
		}

		public void RemoveHarvestUsed( HarvestUsed harvestUsed )
		{
			m_HarvestUseds.Remove( harvestUsed );
		}

		public void Begin( Mobile from, Item item )
		{
			from.Target = new InternalTarget( this, item );
		}

		private class InternalTarget : Target
		{
			private Item m_Item;
			private HarvestSystem m_HarvestSystem;

			public InternalTarget( HarvestSystem harvestSystem, Item item ) : base( 1, true, TargetFlags.None )
			{
				m_HarvestSystem = harvestSystem;
				m_Item = item;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( m_Item.Deleted ) return;

				HarvestItem harvestItem = null;
				int ItemID = 0;
				int X = 0;
				int Y = 0;
				int Z = 0;

				if ( targeted is StaticTarget )
				{
					StaticTarget staticTarget = ( StaticTarget )targeted;

					harvestItem = m_HarvestSystem.HarvestItems.SearchFor( staticTarget.ItemID );

					ItemID = staticTarget.ItemID;
					X = staticTarget.X;
					Y = staticTarget.Y;
					Z = staticTarget.Z;
				}
				
				if ( targeted is LandTarget )
				{
					LandTarget landTarget = ( LandTarget )targeted;

					Tile tile = from.Map.Tiles.GetLandTile(landTarget.Location.X, landTarget.Location.Y);
					int tileID = tile.ID;

					harvestItem = m_HarvestSystem.HarvestItems.SearchFor( tileID );

					ItemID = tileID;
					X = landTarget.X;
					Y = landTarget.Y;
					Z = landTarget.Z;
				}

				if ( m_HarvestSystem.CanHarvest( from, m_Item, harvestItem ) )
				{
					harvestItem.Harvest( from, ItemID, X, Y, Z, m_HarvestSystem );
				}
			}
		}
	}
}