using System;
using System.Collections;
using Server.Misc;
using Server.Spells;
using Server.Spells.First;
using Server.Spells.Second;
using Server.Spells.Third;
using Server.Spells.Fourth;
using Server.Spells.Fifth;
using Server.Spells.Sixth;
using Server.Spells.Seventh;
using Server.Targeting;

namespace Server.Mobiles
{
	public abstract class BaseAggressive : Mobile
	{
		private Timer m_Timer;

		public BaseAggressive()
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();

			base.SpeechHue = Utility.RandomDyedHue();
		}

		public BaseAggressive( Serial serial ) : base( serial )
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public virtual double GetValueFrom( Mobile m )
		{
			return (m.Skills[SkillName.Magery].Value + m.Int) / this.GetDistanceToSqrt( m );
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public virtual void AquireCombatant()
		{
			if ( Deleted )
				return;

			Map map = this.Map;

			if ( map != null )
			{
				Mobile comb = null;
				double val = double.MinValue;

				IPooledEnumerable eable = map.GetMobilesInRange( this.Location, 6 );

				foreach ( Mobile m in eable )
				{
					if ( m != this && this.CanSee( m ) && this.InLOS( m ) && this.CanBeHarmful( m ) )
					{
						double theirVal = GetValueFrom( m );

						if ( theirVal > val )
						{
							comb = m;
							val = theirVal;
						}
					}
				}

				eable.Free();

				this.Combatant = comb;
			}
		}

		protected override void OnTargetChange()
		{
			Target t = this.Target;

			if ( t != null )
			{
				if ( this.Combatant != null && !InRange( this.Combatant, 1 ) )
					RunTo( this.Combatant );

				bool isDispel = t is DispelSpell.InternalTarget;
				bool isParalyze = t is ParalyzeSpell.InternalTarget;

				Mobile toTarget;

				if ( isDispel )
				{
					toTarget = FindDispelTarget( false );

					if ( toTarget != null )
						RunFrom( toTarget );
				}
				else if ( isParalyze )
				{
					toTarget = FindDispelTarget( true );

					if ( toTarget == null )
					{
						toTarget = Combatant;

						if ( toTarget != null )
							RunTo( toTarget );
					}
					else
					{
						RunFrom( toTarget );
					}
				}
				else
				{
					toTarget = Combatant;

					if ( toTarget != null )
						RunTo( toTarget );
				}

				if ( (t.Flags & TargetFlags.Harmful) != 0 && toTarget != null )
				{
					if ( (t.Range == -1 || InRange( toTarget, t.Range )) && CanSee( toTarget ) && InLOS( toTarget ) )
					{
						t.Invoke( this, toTarget );
					}
					else if ( isDispel )
					{
						t.Cancel( this, TargetCancelType.Canceled );
					}
				}
				else if ( (t.Flags & TargetFlags.Beneficial) != 0 )
				{
					t.Invoke( this, this );
				}
				else
				{
					t.Cancel( this, TargetCancelType.Canceled );
				}
			}
		}

		public Mobile FindDispelTarget( bool activeOnly )
		{
			if ( Deleted || CanDispel( this ) )
				return null;

			Map map = this.Map;

			if ( map != null )
			{
				Mobile active = null, inactive = null;
				double actPrio = 0.0, inactPrio = 0.0;

				IPooledEnumerable eable = map.GetMobilesInRange( this.Location, 12 );

				foreach ( Mobile m in eable )
				{
					if ( m != this && CanDispel( m ) )
					{
						double prio = GetDistanceToSqrt( m );

						if ( !activeOnly && (inactive == null || prio < inactPrio) )
						{
							inactive = m;
							inactPrio = prio;
						}

						if ( (Combatant == m || m.Combatant == this) && (active == null || prio < actPrio) )
						{
							active = m;
							actPrio = prio;
						}
					}
				}

				eable.Free();

				return active != null ? active : inactive;
			}

			return null;
		}

		public bool CanDispel( Mobile m )
		{
			return CanBeHarmful( m, false ) && m.GetType().IsDefined( typeof( DispellableAttribute ), false );
		}

		public void Idle()
		{
			if ( Utility.Random( 0, 4 ) == 0 )
				Direction = (Direction)Utility.Random( 8 );
			else if ( !Move( Direction ) )
				Direction = (Direction)Utility.Random( 8 );
		}

		public void RunTo( Mobile m )
		{
			if ( m.Paralyzed )
			{
				if ( InRange( m, 1 ) )
					RunFrom( m );
				else if ( !InRange( m, 2 ) )
					Run( GetDirectionTo( m ) );
			}
			else
			{
				if ( !InRange( m, 1 ) )
					Run( GetDirectionTo( m ) );
			}
		}

		public void RunFrom( Mobile m )
		{
			Run( (GetDirectionTo( m ) - 4) & Direction.Mask );
		}

		public void Run( Direction d )
		{
			Direction = d | Direction.Running;

			if ( !Move( Direction ) )
			{
				int v = (int)d;
				v += (Utility.Random( 0, 1 ) * 2) - 1;
				v &= 0x7;

				Direction = (Direction)v | Direction.Running;

				if ( !Move( Direction ) )
				{
					Mobile c = Combatant;

					if ( c != null )
					{
						AquireCombatant();

						if ( Combatant == null )
							Combatant = c;
					}
				}
			}
		}

		private class AITimer : Timer
		{
			private BaseAggressive m_Owner;
			private DateTime m_NextCastTime;
			private int m_Combo = -1;

			public AITimer( BaseAggressive owner ) : base( TimeSpan.FromSeconds( 1.0 ), TimeSpan.FromSeconds( 1.0 ) )
			{
				this.Priority = TimerPriority.FiftyMS;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				if ( m_Owner.Combatant == null || m_Owner.Combatant.Deleted || !m_Owner.CanSee( m_Owner.Combatant ) || !m_Owner.CanBeHarmful( m_Owner.Combatant, false ) )
				{
					m_Owner.Combatant = null;

					m_Owner.AquireCombatant();

					if ( m_Owner.Combatant != null && !m_Owner.InRange( m_Owner.Combatant, 1 ) )
					{
						m_Owner.RunTo( m_Owner.Combatant );
					}
					else
					{
						if ( m_Owner.Target != null )
							m_Owner.Target.Cancel( m_Owner, TargetCancelType.Canceled );

						Spell spell = null;

						if ( m_Owner.Poisoned )
							spell = new CureSpell( m_Owner, null );
						else if ( m_Owner.Hits < (m_Owner.HitsMax - 50) )
							spell = new GreaterHealSpell( m_Owner, null );
						else if ( m_Owner.Hits < (m_Owner.HitsMax - 20) )
							spell = new HealSpell( m_Owner, null );

						if ( spell == null || !spell.Cast() )
						{
							m_Owner.Idle();
						}
					}
				}
				else
				{
					if ( !m_Owner.StunReady && m_Owner.Skills[SkillName.Wrestling].Value >= 80.0 && m_Owner.Skills[SkillName.Anatomy].Value >= 80.0 )
						EventSink.InvokeStunRequest( new StunRequestEventArgs( m_Owner ) );

					Mobile c = m_Owner.Combatant;

					if ( !m_Owner.InRange( c, 20 ) )
					{
						m_Owner.AquireCombatant();

						if ( m_Owner.Combatant == null && m_Owner.InRange( c, 40 ) )
							m_Owner.Combatant = c;

						c = m_Owner.Combatant;

						if ( c == null )
							return;
					}

					if ( m_Owner.Target != null )
					{
						Mobile toTarget;

						if ( m_Owner.Target is ParalyzeSpell.InternalTarget )
						{
							toTarget = m_Owner.FindDispelTarget( true );

							if ( toTarget == null )
							{
								toTarget = c;
								m_Owner.RunTo( c );
							}
							else if ( m_Owner.InRange( toTarget, 10 ) )
							{
								m_Owner.RunFrom( toTarget );
							}
							else if ( !m_Owner.InRange( toTarget, 12 ) )
							{
								m_Owner.RunTo( toTarget );
							}
						}
						else
						{
							toTarget = c;
							m_Owner.RunTo( c );
						}

						if ( (m_Owner.Target.Range == -1 || m_Owner.InRange( toTarget, m_Owner.Target.Range )) && m_Owner.CanSee( toTarget ) && m_Owner.InLOS( toTarget ) )
							m_Owner.Target.Invoke( m_Owner, toTarget );
					}
					else if ( DateTime.Now > m_NextCastTime && m_Owner.Spell == null && m_Owner.InRange( c, 12 ) )
					{
						Spell spell = null;

						Mobile toDispel = m_Owner.FindDispelTarget( true );

						if ( m_Owner.Poisoned )
						{
							spell = new CureSpell( m_Owner, null );
						}
						else if ( toDispel != null )
						{
							if ( Utility.Random( 0, 4 + (m_Owner.Hits == 0 ? m_Owner.HitsMax : (m_Owner.HitsMax / m_Owner.Hits)) ) >= 4 )
							{
								if ( m_Owner.Hits < (m_Owner.HitsMax - 50) )
									spell = new GreaterHealSpell( m_Owner, null );
								else if ( m_Owner.Hits < (m_Owner.HitsMax - 20) )
									spell = new HealSpell( m_Owner, null );
							}

							if ( spell == null )
							{
								if ( Utility.Random( 2 ) == 0 && !toDispel.Paralyzed && !toDispel.Frozen )
									spell = new ParalyzeSpell( m_Owner, null );
								else
									spell = new DispelSpell( m_Owner, null );
							}
						}
						else if ( m_Combo != -1 )
						{
							if ( m_Combo == 0 )
							{
								spell = new ExplosionSpell( m_Owner, null );
								++m_Combo;
							}
							else if ( m_Combo == 1 )
							{
								spell = new WeakenSpell( m_Owner, null );
								++m_Combo;
							}
							else if ( m_Combo == 2 )
							{
								if ( !c.Poisoned )
									spell = new PoisonSpell( m_Owner, null );

								++m_Combo;
							}

							if ( m_Combo == 3 && spell == null )
							{
								switch ( Utility.Random( 3 ) )
								{
									default:
									case 0:
									{
										if ( c.Int < c.Dex )
											spell = new FeeblemindSpell( m_Owner, null );
										else
											spell = new ClumsySpell( m_Owner, null );

										++m_Combo;

										break;
									}
									case 1:
									{
										spell = new EnergyBoltSpell( m_Owner, null );
										m_Combo = -1;
										break;
									}
									case 2:
									{
										spell = new FlameStrikeSpell( m_Owner, null );
										m_Combo = -1;
										break;
									}
								}
							}
							else if ( m_Combo == 4 && spell == null )
							{
								spell = new MindBlastSpell( m_Owner, null );
								m_Combo = -1;
							}
						}
						else
						{
							if ( (c.Spell is HealSpell || c.Spell is GreaterHealSpell) && !c.Poisoned )
							{
								spell = new PoisonSpell( m_Owner, null );
							}
							else
							{
								switch ( Utility.Random( 4 + (m_Owner.Hits == 0 ? m_Owner.HitsMax : (m_Owner.HitsMax / m_Owner.Hits)) ) )
								{
									default:
									case 0:
									{
										if ( m_Owner.Hits < (m_Owner.HitsMax - 50) )
											spell = new GreaterHealSpell( m_Owner, null );
										else if ( m_Owner.Hits < (m_Owner.HitsMax - 20) )
											spell = new HealSpell( m_Owner, null );

										break;
									}
									case 1:
									{
										if ( !c.Poisoned )
											spell = new PoisonSpell( m_Owner, null );

										break;
									}
									case 2:
									{
										switch ( Utility.Random( 8 ) )
										{
											default:
											case 0: spell = new MagicArrowSpell( m_Owner, null ); break;
											case 1: spell = new HarmSpell( m_Owner, null ); break;
											case 2: spell = new FireballSpell( m_Owner, null ); break;
											case 3: spell = new LightningSpell( m_Owner, null ); break;
											case 4: spell = new MindBlastSpell( m_Owner, null ); break;
											case 5: spell = new EnergyBoltSpell( m_Owner, null ); break;
											case 6: spell = new ExplosionSpell( m_Owner, null ); break;
											case 7: spell = new FlameStrikeSpell( m_Owner, null ); break;
										}

										break;
									}
									case 3:
									{
										if ( m_Owner.Mana > 60 )
										{
											if ( Utility.Random( 5 ) == 0 && !m_Owner.Paralyzed && !m_Owner.Frozen && !m_Owner.Poisoned )
											{
												m_Combo = 0;
												spell = new ParalyzeSpell( m_Owner, null );
											}
											else
											{
												m_Combo = 1;
												spell = new ExplosionSpell( m_Owner, null );
											}
										}

										break;
									}
								}
							}
						}

						if ( toDispel != null )
						{
							if ( m_Owner.InRange( toDispel, 10 ) )
								m_Owner.RunFrom( toDispel );
							else if ( !m_Owner.InRange( toDispel, 12 ) )
								m_Owner.RunTo( toDispel );
						}
						else
						{
							m_Owner.RunTo( c );
						}

						if ( spell != null && spell.Cast() )
						{
							m_NextCastTime = DateTime.Now + TimeSpan.FromSeconds( 0.18 );
						}
					}
					else
					{
						m_Owner.RunTo( c );
					}
				}

				if ( m_Owner.Combatant == null )
				{
					this.Interval = TimeSpan.FromSeconds( 1.0 );
					this.Priority = TimerPriority.FiftyMS;
				}
				else
				{
					this.Interval = TimeSpan.FromSeconds( 0.2 );
					this.Priority = TimerPriority.TwentyFiveMS;
				}
			}
		}
	}
}