using System;
using System.Collections;
using Server.Items;
using Server.Network;
using Server.ContextMenus;
using Server.Mobiles;

namespace Server.Mobiles
{
	public abstract class BaseVendor : Mobile, IVendor
	{
		private DateTime m_LastRestock;

		public BaseVendor( string title )
		{
			this.Body = 0x0190;
			this.Title = title;

			Container pack;
			//these packs MUST exist, or the client will crash when the packets are sent
			pack = new Container( 0x9B2, 0x3C, 0x48, new Rectangle2D( 44, 65, 142, 94 ) );//stuff for a regular backpack
			pack.Layer = Layer.ShopBuy;
			pack.Newbied = true;
			AddItem( pack );

			pack = new Container( 0x9B2, 0x3C, 0x48, new Rectangle2D( 44, 65, 142, 94 ) );//stuff for a regular backpack
			pack.Layer = Layer.ShopResale;
			pack.Newbied = true;
			AddItem( pack );
		}

		public BaseVendor( Serial serial ) : base( serial )
		{
		}

		public DateTime LastRestock
		{
			get
			{
				return m_LastRestock;
			}
			set
			{
				m_LastRestock = value;
			}
		}

		public virtual TimeSpan RestockDelay
		{
			get
			{
				return TimeSpan.FromMinutes( 15.0 );
			}
		}

		public Container BuyPack
		{
			get
			{
				Container pack = FindItemOnLayer( Layer.ShopBuy ) as Container;
				if ( pack == null )
				{
					pack = new Container( 0x9B2, 0x3C, 0x48, new Rectangle2D( 44, 65, 142, 94 ) );//stuff for a regular backpack
					pack.Layer = Layer.ShopBuy;
					AddItem( pack );
				}

				return pack;
			}
		}

		public virtual void Restock()
		{
			IBuyItemInfo[] buyInfo = this.GetBuyInfo();

			foreach ( IBuyItemInfo bii in buyInfo )
				bii.OnRestock();
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			Mobile from = e.Mobile;

			if ( from.InRange( this, 1 ) )
			{
				if ( e.HasKeyword( 0x014D ) || e.HasKeyword( 0x177 ) ) // *vendor sell* || *sell*
				{
					VendorSell( from );
				}
				else if ( e.HasKeyword( 0x171 ) )//*buy*
				{
					VendorBuy( from );
				}
			}
		}

		public virtual void VendorBuy( Mobile from )
		{
			int count = 0;
			ArrayList list;
			IBuyItemInfo[] buyInfo = this.GetBuyInfo();
			IShopSellInfo[] sellInfo = this.GetSellInfo();

			list = new ArrayList( buyInfo.Length );

			foreach ( IBuyItemInfo buyItem in buyInfo )
			{
				if ( buyItem.Amount > 0 )
				{
					list.Add( new BuyItemState( buyItem.Name, BuyPack.Serial, 0x7FFFFFFF - count, buyItem.Price, buyItem.Amount, buyItem.ItemID, buyItem.Hue ) );
					count++;
				}
			}

			Container cont = this.BuyPack;
			if ( cont != null )
			{
				foreach ( Item item in cont.Items )
				{
					int price = 0;
					string name = "";

					foreach( IShopSellInfo ssi in sellInfo )
					{
						if ( ssi.IsSellable( item ) )
						{
							price = ssi.GetBuyPriceFor( item );
							name = ssi.GetNameFor( item );
							break;
						}
					}

					list.Add( new BuyItemState( name, BuyPack.Serial, item.Serial, price, item.Amount, item.ItemID, item.Hue ) );
					count++;
				}
			}

			//one (not all) of the packets uses a byte to describe number of items in the list.  Osi = dumb.
			if ( list.Count > 255 )
				Console.WriteLine( "Vendor Warning: Vendor {0} has more than 255 buy items, may cause client errors!", this );

			if ( list.Count > 0 )
			{
				list.Sort( new BuyItemStateComparer() );

				from.Send( new VendorBuyContent( list ) );
				from.Send( new VendorBuyList( this, list ) );
				from.Send( new DisplayBuyList( this ) );
				from.Send( new MobileStatusExtended( from ) );
			}
		}

		public virtual void VendorSell( Mobile from )
		{
			Container pack = from.Backpack;

			if ( pack != null )
			{
				IShopSellInfo[] info = GetSellInfo();

				Hashtable table = new Hashtable();

				foreach ( IShopSellInfo ssi in info )
				{
					Item[] items = pack.FindItemsByType( ssi.Types );

					foreach ( Item item in items )
					{
						if ( !item.Newbied && item.Movable && ssi.IsSellable( item ) )
							table[item] = new SellItemState( item, ssi.GetSellPriceFor( item ), ssi.GetNameFor( item ) );
					}
				}

				from.Send( new VendorSellList( this, table ) );
			}
		}

		public virtual bool OnBuyItems( Mobile buyer, ArrayList list )
		{
			IBuyItemInfo[] buyInfo = this.GetBuyInfo();
			IShopSellInfo[] info = GetSellInfo();
			int totalCost = 0;
			ArrayList validBuy = new ArrayList( list.Count );
			Container cont;
			bool bought = false;
			bool fromBank = false;

			foreach ( BuyItemResponse buy in list )
			{
				Serial ser = 0x7FFFFFFF - buy.Serial;
				int amount = buy.Amount;
				if ( ser >= 0 && ser <= buyInfo.Length )
				{
					IBuyItemInfo bii = buyInfo[ser];
					if ( amount > bii.Amount )
						amount = bii.Amount;
					if ( amount <= 0 )
						continue;

					totalCost += bii.Price * amount;
					validBuy.Add( buy );
				}
				else
				{
					Item item = World.FindItem( buy.Serial );

					if ( item == null || item.RootParent != this )
						continue;

					if ( amount > item.Amount )
						amount = item.Amount;
					if ( amount <= 0 )
						continue;

					foreach( IShopSellInfo ssi in info )
					{
						if ( ssi.IsSellable( item ) )
						{
							if ( ssi.IsResellable( item ) )
							{
								totalCost += ssi.GetBuyPriceFor( item );
								validBuy.Add( buy );
								break;
							}
						}
					}
				}
			}//foreach

			cont = buyer.Backpack;
			if ( cont != null )
			{
				if ( cont.ConsumeTotal( typeof( Gold ), totalCost ) )
					bought = true;
				else if ( totalCost < 2000 )
					PublicOverheadMessage( MessageType.Regular, 0, 500192 );//Begging thy pardon, but thou casnt afford that,
			}

			if ( !bought && totalCost >= 2000 )
			{
				cont = buyer.BankBox;
				if ( cont.ConsumeTotal( typeof( Gold ), totalCost ) )
				{
					bought = true;
					fromBank = true;
				}
				else
				{
					PublicOverheadMessage( MessageType.Regular, 0, 500191 );//Begging thy pardon, but thy bank account lacks these funds.
				}
			}

			if ( !bought )
				return false;

			cont = buyer.Backpack;
			if ( cont == null )
				cont = buyer.BankBox;

			foreach ( BuyItemResponse buy in validBuy )
			{
				Serial ser = 0x7FFFFFFF - buy.Serial;
				int amount = buy.Amount;

				if ( ser >= 0 && ser <= buyInfo.Length )
				{
					IBuyItemInfo bii = buyInfo[ser];

					if ( amount > bii.Amount )
						amount = bii.Amount;

					bii.Amount -= amount;
					Item item = bii.GetItem();
					if ( item.Stackable )
					{
						item.Amount = amount;

						if ( !cont.TryDropItem( buyer, item, false ) )
							item.MoveToWorld( buyer.Location, buyer.Map );
					}
					else
					{
						item.Amount = 1;
						if ( !cont.TryDropItem( buyer, item, false ) )
							item.MoveToWorld( buyer.Location, buyer.Map );

						for (int i=1;i<amount;i++)
						{
							item = bii.GetItem();
							item.Amount = 1;
							if ( !cont.TryDropItem( buyer, item, false ) )
								item.MoveToWorld( buyer.Location, buyer.Map );
						}
					}
				}
				else
				{
					Item item = World.FindItem( buy.Serial );

					if ( item == null )
						continue;

					if ( amount > item.Amount )
						amount = item.Amount;

					foreach( IShopSellInfo ssi in info )
					{
						if ( ssi.IsSellable( item ) )
						{
							if ( ssi.IsResellable( item ) )
							{
								Item buyItem;
								if ( amount >= item.Amount )
								{
									buyItem = item;
								}
								else
								{
									buyItem = item.Dupe( amount );
									item.Amount -= amount;
								}

								if ( !cont.TryDropItem( buyer, buyItem, false ) )
									buyItem.MoveToWorld( buyer.Location, buyer.Map );

								break;
							}
						}
					}
				}
			}//foreach

			if ( fromBank )
				PrivateOverheadMessage( MessageType.Regular, 0, true, string.Format( "The total of thy purchase is {0} gold, which has been withdrawn from your bank account.  My thanks for thy patronage.", totalCost ), buyer.NetState );
			else
				PrivateOverheadMessage( MessageType.Regular, 0, true, string.Format( "The total of thy purchase is {0} gold.  My thanks for thy patronage.", totalCost ), buyer.NetState );

			return true;
		}

		public virtual bool OnSellItems( Mobile seller, ArrayList list )
		{
			IShopSellInfo[] info = GetSellInfo();
			IBuyItemInfo[] buyInfo = this.GetBuyInfo();
			int GiveGold = 0;
			int Sold = 0;
			Container cont;

			foreach ( SellItemResponse resp in list )
			{
				if ( resp.Item.RootParent != seller || resp.Amount <= 0 )
					continue;

				foreach( IShopSellInfo ssi in info )
				{
					if ( ssi.IsSellable( resp.Item ) )
					{
						int amount = resp.Amount;
						if ( amount > resp.Item.Amount )
							amount = resp.Item.Amount;

						if ( ssi.IsResellable( resp.Item ) )
						{
							bool found = false;
							foreach ( IBuyItemInfo bii in buyInfo )
							{
								if ( bii.Restock( resp.Item ) )
								{
									found = true;
									if ( resp.Item.Amount - resp.Amount <= 0 )
										resp.Item.Delete();
									break;
								}
							}

							//not in their buy list, but it is resellable, so put it in the resale pack
							if ( !found )
							{
								cont = this.BuyPack;
								if ( resp.Amount < resp.Item.Amount )
								{
									resp.Item.Amount -= resp.Amount;
									if ( cont != null )
										cont.DropItem( resp.Item.Dupe( resp.Amount ) );
								}
								else
								{
									if ( cont != null )
										cont.DropItem( resp.Item );
								}
							}
						}
						else
						{
							//we won't
							resp.Item.Delete();
						}

						Sold++;
						GiveGold += ssi.GetSellPriceFor( resp.Item )*amount;
						break;
					}
				}
			}

			if ( Sold > 0 )
			{
				if ( GiveGold > 0 )
				{
					Item gold;
					cont = seller.Backpack;
					if ( cont != null )
					{
						gold = cont.FindItemByType( typeof( Gold ) );
						if ( gold != null )
						{
							gold.Amount += GiveGold;
						}
						else 
						{
							gold = new Gold( GiveGold );
							if ( !cont.TryDropItem( seller, gold, true ) )
								gold.MoveToWorld( seller.Location, seller.Map );
						}
					}
					else
					{
						gold = new Gold( GiveGold );
						gold.MoveToWorld( seller.Location, seller.Map );
					}
					seller.PlaySound( 0x0037 );//Gold dropping sound
				}
				//no cliloc for this?
				PrivateOverheadMessage( MessageType.Regular, 0, false, string.Format("Thank you! I bought {0} item{1}. Here is your {2}gp.", Sold, (Sold > 1 ? "s" : ""), GiveGold ), seller.NetState );
			}

			return true;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void GetContextMenuEntries( Mobile from, ArrayList list )
		{
			base.GetContextMenuEntries( from, list );

			list.Add( new VendorBuyEntry( from, this ) );
			list.Add( new VendorSellEntry( from, this ) );
		}


		public abstract IShopSellInfo[] GetSellInfo();
		public abstract IBuyItemInfo[] GetBuyInfo();
	}
}

namespace Server.ContextMenus
{
	public class VendorBuyEntry : ContextMenuEntry
	{
		private BaseVendor m_Vendor;

		public VendorBuyEntry( Mobile from, BaseVendor vendor ) : base( 6103, from.InRange( vendor, 8 ) )
		{
			m_Vendor = vendor;
		}

		public override void OnClick()
		{
			if ( this.Owner.From.InRange( m_Vendor, 8 ) )
			{
				m_Vendor.VendorBuy( this.Owner.From );
			}
		}
	}

	public class VendorSellEntry : ContextMenuEntry
	{
		private BaseVendor m_Vendor;

		public VendorSellEntry( Mobile from, BaseVendor vendor ) : base( 6104, from.InRange( vendor, 8 ) )
		{
			m_Vendor = vendor;
		}

		public override void OnClick()
		{
			if ( this.Owner.From.InRange( m_Vendor, 8 ) )
			{
				m_Vendor.VendorSell( this.Owner.From );
			}
		}
	}
}
